import { select } from 'd3';
import { viz } from './viz';
import { observeResize } from '@drbagwell/responsive-viz-axes';
import { FFT } from './fftClass';
import { data } from '@drbagwell/tremor-analysis-data';

export const main = (container, { state, setState }) => {
  const dimensions = observeResize({
    state,
    setState,
    container,
  });

  if (dimensions === null) return;

  const { width, height } = dimensions;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  const combinedFFTSignal = [];
  const nPoints = 512;

  for (let i = 0; i < nPoints; i++) {
    combinedFFTSignal.push(data[i].heartRate);
  }

  // FFT next
  const runFFT = new FFT(nPoints); // should be 512
  const out = runFFT.createComplexArray();
  runFFT.realTransform(out, combinedFFTSignal);
  //runFFT.completeSpectrum(out); // this produces double sided spectrum
  const interleavedValues = out; // Example interleaved data
  const numComplexValues = interleavedValues.length / 2;

  const realArray = interleavedValues.slice(
    0,
    numComplexValues,
  );
  const imagArray = interleavedValues.slice(
    numComplexValues,
  );

  // Assuming realArray and imagArray contain corresponding real and imaginary values
  const magnitudeArray = realArray.map((a, i) =>
    Math.sqrt(a * a + imagArray[i] * imagArray[i]),
  );
  const phaseArray = realArray.map((a, i) =>
    Math.atan2(imagArray[i], a),
  );

  const fs = 62.5; // 62.5 samples per second
  const fnyquist = fs / 2; //Nyquist frequency

  let plotArray = [];
  for (let i = 0; i < magnitudeArray.length; i++) {
    plotArray.push({
      indx: i * (62.5 / 128),
      sFreq: magnitudeArray[i],
    });
  }

  viz(svg, {
    data: plotArray,
    xValue: (d) => d.indx,
    xAxisLabelText: 'Frequency (Bins)',
    xAxisLabelOffset: 35,
    yValue: (d) => d.sFreq,
    yAxisLabelText: 'Raw Magnitude',
    yAxisLabelOffset: 20,
    circleRadius: 500 / 100,
    circleOpacity: 569 / 1000,
    marginTop: 90,
    marginRight: 50,
    marginBottom: 50,
    marginLeft: 50,
    width,
    height,
    innerRectFill: '#B5B5B5',
  });
};
